/*
// Created by Tiny on 3/25/2021.
*/

#include "collision.h"



//Collisions Functions

/*!
 * \fn int test_position(Personnage * pers, Prof * prof, DS * exam, MAP * map)
 * \brief This function calculates if the there is a collision between the character and anything else.
 *
 * \param [in] pers a pointer on the character
 * \param [in] prof a pointer on the teacher
 * \param [in] exam a pointer on the exam
 * \param [in] map a pointer on the map
 * \return 0 if there is nothing, 1 if there's a wall, 2 if it's an exam, 3 if it's a teacher, 4 if the character is trying to to out the window.
 */

int test_position(Personnage * pers, Prof * prof, DS * exam, MAP * map){
    //test variables
    int testbounds = test_bounds_col(pers); ;
    int testprof = test_prof_col(pers,prof);

    //test bounds collision

    if(testbounds) return 4;

    //test prof collision

    if(testprof) return 3;

    //return the map test position

    return test_map_col(pers,map,exam); // = 0 if nothing , 1 if wall, 2 if exam

}
/*!
 * \fn int test_bounds_col(Personnage * pers)
 * \brief This function calculates if the character is trying to go out the window.
 *
 * \param [in] pers a pointer on the character
 * \return 1 if the character is trying to go out the window, 0 otherwise.
 */

int test_bounds_col(Personnage * pers){
    if((pers->x_pers <= 0) || (pers->y_pers <= 0) || (pers->x_pers >= WINDOW_WIDTH - pers->larg_pers) || (pers->y_pers >= WINDOW_HEIGHT - pers->long_pers)) {
        return 1;
    }
    return 0;
}

/*!
 * \fn int test_prof_col(Personnage * pers, Prof * prof)
 * \brief This function calculates if the character is in collision with the teacher.
 *
 * \param [in] pers a pointer on the character
 * \param [in] prof a pointer on the teacher
 * \return 1 if there is a collision, 0 otherwise.
 */

int test_prof_col(Personnage * pers, Prof * prof){

    int testhl;
    int testhr;
    int testll;
    int testlr;


    testhl=test_point_with_prof(pers->x_pers, pers->y_pers,prof);
    testhr=test_point_with_prof(pers->x_hr, pers->y_hr,prof) ;
    testll=test_point_with_prof(pers->x_ll, pers->y_ll,prof);
    testlr=test_point_with_prof(pers->x_lr, pers->y_lr,prof) ;

    if(testhl){
        return 1;
    }
    else if (testhr){
        return 1;
    }
    else if (testll){
        return 1;
    }
    else if (testlr){
        return 1;
    }
    return 0;
}

/*!
 * \fn int test_point_with_prof(int x, int y, Prof *prof)
 * \brief This function tests if one of the character's corner is in collision with the teacher.
 *
 * \param [in] x one of the character's abscissa (one of the 4 corners)
 * \param [in] y the ordinate corresponding to the same corner
 * \param [in] prof a pointer on the teacher
 * \return 1 if there is a collision, 0 otherwise.
 */

int test_point_with_prof(int x, int y, Prof *prof){
    if (( x > prof->x_prof) && (x<prof->x_prof+prof->larg_prof)) {
        if (( y > prof->y_prof) && (y < prof->y_prof + prof->long_prof)) {
            return 1;
        }
    }
    return 0;
}

/*!
 * \fn int test_map_col(Personnage * pers,  MAP * map, DS * exam )
 * \brief This function tests if the character is in collision with the teacher.
 *
 * \param [in] pers a pointer on the character
 * \param [in] map a pointer on the map
 * \param [in] exam a pointer on an exam
 * \return 2 if it's an exam, 1 if there's a wall, 0 otherwise.
 */

int test_map_col(Personnage * pers,  MAP * map, DS * exam ){
    int testhl = test_point_with_map(pers->x_pers,pers->y_pers,map,exam);
    int testhr = test_point_with_map(pers->x_hr,pers->y_hr,map,exam) ;
    int testll = test_point_with_map(pers->x_ll,pers->y_ll,map,exam) ;
    int testlr = test_point_with_map(pers->x_lr,pers->y_lr,map,exam) ;

    if(testhl==2 || testhr==2 || testll==2 || testlr==2) {
        return 2;
    }
    else if(testhl==1 || testhr==1 || testll==1 || testlr==1){
        return 1;
    }
    else return 0;
}

/*!
 * \fn test_point_with_map(int x, int y, MAP *map, DS * exam)
 * \brief This function tests if one of the character's corner is in collision with an exam, a wall or neither.
 *
 * \param [in] x one of the character's abscissa (one of the 4 corners)
 * \param [in] y the ordinate corresponding to the same corner
 * \param [in] map a pointer on the map
 * \param [in] exam a pointer on the exam
 * \return 2 if theres is a collision with an exam, 1 with a wall, 0 otherwise.
 */
int test_point_with_map(int x, int y, MAP *map, DS * exam){
    if (map->functional[y][x] == 2){         //if zone exams
        //test exam collision
        if ((x > exam->x_DS) && (x < exam->x_DS+exam->larg_DS)) {
            if(((y > exam->y_DS) && (y < exam->y_DS+exam->long_DS))){
                return 2;
            }
        }
    }
    else if (map->functional[y][x] == 1){         //if zone wall
        return 1;
    }
    return 0;
}


int test_position_prof(Prof * prof,MAP * map, DS *exam){
    int testbound = test_bounds_prof(prof); //0 if nothing,  1 if bounds
    int testwall = test_map_col_prof(prof,map,exam); //0 if nothing, 1 if wall
    if(testbound==1){
        return 1;
    }
    else if(testwall == 1){
        return 1;
    }
    return 0;
}

int test_bounds_prof(Prof * prof){
    if((prof->x_prof <= 0) || (prof->y_prof <= 0) || (prof->x_prof >= WINDOW_WIDTH - prof->larg_prof) || (prof->y_prof >= WINDOW_HEIGHT - prof->long_prof-20)) {
        return 1;
    }
    return 0;
}

int test_map_col_prof(Prof * prof,  MAP * map, DS * exam){
    int testhl = test_point_with_map(prof->x_prof,prof->y_prof,map,exam);
    int testhr = test_point_with_map(prof->x_hr,prof->y_hr,map,exam) ;
    int testll = test_point_with_map(prof->x_ll,prof->y_ll,map,exam) ;
    int testlr = test_point_with_map(prof->x_lr,prof->y_lr,map,exam) ;
    if(testhl==1 || testhr==1 || testll==1 || testlr==1){
        return 1;
    }
    else return 0;
}